using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
namespace id.Quake.Files {
	public partial class Level {

		#region Properties

		private readonly VisibilityCluster[] visibilityClusters;
		/// <summary>Gets the visibility clusters level.</summary>
		[Category("BSP Tree")]
		public VisibilityCluster[] VisibilityClusters {
			get { return this.visibilityClusters; }
		}

		#endregion

		/// <summary>Represents an visibility list cluster.</summary>
		[TypeConverter(typeof(ExpandableObjectConverter))]
		public class VisibilityCluster {

			private readonly Level level;
			/// <summary>Gets the level that contains this cluster information.</summary>
			[Browsable(false)]
			public Level Level {
				get { return this.level; }
			}

			private readonly uint offset;
			/// <summary>Gets the offset to the compressed visibility data from the a of the visibility lump.</summary>
			public uint Offset {
				get { return this.offset; }
			}

			private readonly uint phs;

			internal List<int> visibleLeaves;
			/// <summary>Gets an array of leaf indices that are contained within this cluster.</summary>
			public int[] VisibleLeaves {
				get { return this.visibleLeaves.ToArray(); }
			}

			public VisibilityCluster(Level level, Stream stream) {
				this.level = level;
				this.visibleLeaves = new List<int>();
				BinaryReader WorkingReader = new BinaryReader(stream);
				this.offset = WorkingReader.ReadUInt32();
				this.phs = WorkingReader.ReadUInt32();
			}

			public override string ToString() {
				return string.Format("{0} Visible Lea{1}", this.visibleLeaves.Count, this.visibleLeaves.Count == 1 ? "f" : "ves");
			}

		}

	}
}
