using System;
using System.IO;
using System.ComponentModel;
using System.Text;

namespace id.Quake.Files {

	[SupportedExtension("lmp")]
	public class PictureLump : IPicture, INamedResource {

		#region Properties

		private readonly int width;
		/// <summary>Width, in pixels, of the texture.</summary>
		[Description("Width of the texture in pixels.")]
		public int Width {
			get { return this.width; }
		}

		private readonly int height;
		/// <summary>Height, in pixels, of the texture.</summary>
		[Description("Height of the texture in pixels.")]
		public int Height {
			get { return this.height; }
		}

		private byte[] data;

		private readonly ResourceLoader Resources;

		private string name;
		[Browsable(false)]
		public string Name { get { return this.name; } set { this.name = value; } }

		[Browsable(false)]
		public bool NamedInternally { get { return false; } }

		#endregion

		#region Methods


		#endregion


		#region Constructor

		public PictureLump(ResourceLoader resources, Stream stream) {

			this.Resources = resources;

			BinaryReader PictureReader = new BinaryReader(stream);

			this.width = PictureReader.ReadInt32();
			this.height = PictureReader.ReadInt32();

			this.data = PictureReader.ReadBytes(this.Width * this.Height);

		}

		#endregion

		#region IPicture Members

		public Picture Picture{
			get {
				Palette P = new Palette(this.Resources, this.Resources.OpenFile("gfx/palette.lmp"));
				return new Picture(this.width, this.height, Array.ConvertAll<byte, int>(this.data, delegate(byte b) { return P.Colours[b]; }));
			}
		}

		#endregion
	}
}
