using System;
using System.Collections.Generic;
using QuakeC = id.Quake.Files.QuakeC;
using System.IO;

namespace id.Quake {
	public partial class VirtualMachine {

		private float CurrentTime;
		private float LastTime;
		private float TimeDelta;

		private float Gravity;

		/// <summary>Update the game state.</summary>
		/// <param name="time">The total time elapsed in the game.</param>
		public void Update(float time) {

			LastTime = CurrentTime;
			CurrentTime = time;
			TimeDelta = CurrentTime - LastTime;

			Gravity = this.ConsoleVariables["sv_gravity"].Float;

			// Set "self" and "other" to worldspawn.
			this.Self.Value = this.Edicts[0];
			this.Other.Value = this.Edicts[0];

			// Update the time value.
			this.Time.Value = time;

			// Iterate over edicts:
			foreach (Edict Ed in this.edicts.GetActiveEdicts()) {

				// Work out which physics model to use depending on the movement type:
				switch ((MovementType)(float)Ed.Properties["movetype"]) {
					case MovementType.Toss:
					case MovementType.Bounce:
					case MovementType.Fly:
					case MovementType.FlyMissile:
						Toss(Ed);
						break;

				}
				
			}

			// Run StartFrame()
			this.Invoke("StartFrame");

		}


		#region Physics

		private void Toss(Edict e) {
			Edict.EdictFlags Flags = ((Edict.EdictFlags)(float)e.Properties["flags"]);

			if ((Flags & Edict.EdictFlags.OnGround) != 0) return;

			Vector3 Position = (Vector3)e.Properties["origin"];
			Vector3 Velocity = (Vector3)e.Properties["velocity"];
			Velocity.Z -= TimeDelta * Gravity;
			e.Properties["velocity"] = Velocity;
			e.Properties["origin"] = Position + Velocity * TimeDelta;

		}

		#endregion

	}
}
