using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace Ranger {
	public partial class CinematicPreviewer : Form {

		#region FMOD

		private FMOD.Channel channel = null;
		private FMOD.Sound sound = null;

		private FMOD.CREATESOUNDEXINFO createsoundexinfo = new FMOD.CREATESOUNDEXINFO();

		private FMOD.SOUND_PCMREADCALLBACK pcmreadcallback;
		private FMOD.SOUND_PCMSETPOSCALLBACK pcmsetposcallback;

		private FMOD.RESULT PCMREADCALLBACK(IntPtr soundraw, IntPtr data, uint datalen) {

			byte[] FinalData = new byte[datalen];

			for (int i = 0; i < FinalData.Length; ++i) {
				if (SampleQueueOffset == SampleQueue.Length) {
					SampleQueueOffset = 0;

					id.Quake.Picture Frame;
					while (!this.Cinematic.ReadFrame(out Frame, out SampleQueue)) {
						this.Cinematic.SeekToStart();
					}
					lock (this.PendingFrames) {
						this.PendingFrames.Enqueue(Frame);
					}

				}
				FinalData[i] = this.SampleQueue[SampleQueueOffset++];
			}

			Marshal.Copy(FinalData, 0, data, FinalData.Length);

			this.Invalidate();

			return FMOD.RESULT.OK;
		}

		private FMOD.RESULT PCMSETPOSCALLBACK(IntPtr soundraw, int subsound, uint pcmoffset, FMOD.TIMEUNIT postype) {
			return FMOD.RESULT.OK;
		}

		private byte[] SampleQueue = new byte[0];
		private int SampleQueueOffset;


		private Queue<id.Quake.Picture> PendingFrames;

		#endregion

	

		private id.Quake.Files.Cinematic Cinematic;

		private Bitmap CinematicSurface;
		private Rectangle CinematicSize;

		public CinematicPreviewer(string title, id.Quake.Files.Cinematic cinematic) {
			InitializeComponent();
			this.Text = title;
			Cinematic = cinematic;



			CinematicSurface = new Bitmap(this.Cinematic.Width, this.Cinematic.Height, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
			CinematicSize = new Rectangle(0, 0, this.Cinematic.Width, this.Cinematic.Height);
			this.BackgroundImage = CinematicSurface;
			this.ClientSize = new Size(this.Cinematic.Width, this.Cinematic.Height);
			this.PendingFrames = new Queue<id.Quake.Picture>();
			this.Disposed += new EventHandler(CinematicPreviewer_Disposed);
			Application.Idle += new EventHandler(Application_Idle);

			if (cinematic.SampleWidth == 1 || cinematic.SampleWidth == 2) {

				this.pcmreadcallback = new FMOD.SOUND_PCMREADCALLBACK(this.PCMREADCALLBACK);
				this.pcmsetposcallback = new FMOD.SOUND_PCMSETPOSCALLBACK(this.PCMSETPOSCALLBACK);

				// Ack...
				uint frequency = (uint)cinematic.SampleRate;
				uint channels = (uint)cinematic.Channels;
				createsoundexinfo.cbsize = Marshal.SizeOf(createsoundexinfo);
				createsoundexinfo.fileoffset = 0;
				createsoundexinfo.length = 1024;
				createsoundexinfo.numchannels = (int)channels;
				createsoundexinfo.defaultfrequency = (int)frequency;
				createsoundexinfo.format = cinematic.SampleWidth == 1 ? FMOD.SOUND_FORMAT.PCM8 : FMOD.SOUND_FORMAT.PCM16;
				createsoundexinfo.pcmreadcallback = pcmreadcallback;
				createsoundexinfo.pcmsetposcallback = pcmsetposcallback;
				createsoundexinfo.dlsname = null;

				Program.FmodErrorCheck(Program.SoundSystem.createSound((string)null, 
					FMOD.MODE._2D | FMOD.MODE.DEFAULT | FMOD.MODE.OPENUSER | FMOD.MODE.LOOP_NORMAL | FMOD.MODE.SOFTWARE | FMOD.MODE.CREATESTREAM, 
					ref createsoundexinfo, ref sound));

				Program.FmodErrorCheck(Program.SoundSystem.playSound(FMOD.CHANNELINDEX.FREE, sound, false, ref channel));
			}

		}

		void Application_Idle(object sender, EventArgs e) {
			lock (PendingFrames) {

				id.Quake.Picture Frame = null;
				while (PendingFrames.Count > 0) {
					Frame = PendingFrames.Dequeue();
					break;
				}
				if (Frame != null) {
					BitmapData CopyData = this.CinematicSurface.LockBits(this.CinematicSize, ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
					Marshal.Copy(Frame.Data, 0, CopyData.Scan0, Frame.Data.Length);
					this.CinematicSurface.UnlockBits(CopyData);
					this.Invalidate();
				}
			}
		}


		void CinematicPreviewer_Disposed(object sender, EventArgs e) {
			this.CinematicSurface.Dispose();
		}
	}
}