using System;
using Microsoft.Xna.Framework.Graphics;
using System.Collections.Generic;

using Quake = id.Quake;
using Xna = Microsoft.Xna.Framework;
using System.Runtime.InteropServices;
using System.IO;

namespace XnaQuake.Renderer {
	class ModelRenderer : IDrawable, ILoadable {

		#region Private Members

		private readonly Quake.Files.Model SourceModel;
		private Texture2D DiffuseTexture;
		private Xna.Graphics.VertexPositionNormalTexture[] Vertices;
		private short[] Triangles;

		#endregion

		#region Public Members

		public List<EntityPosition> EntityPositions = new List<EntityPosition>();

		#endregion

		#region Constructor

		public ModelRenderer(Quake.Files.Model model) {
			this.SourceModel = model;
		}

		#endregion

		#region ILoadable Members

		private Xna.GraphicsDeviceManager Graphics;

		public void Load(id.Quake.ResourceLoader resources, Microsoft.Xna.Framework.GraphicsDeviceManager graphics, Xna.Content.ContentManager content) {

			this.Graphics = graphics;
			this.DiffuseTexture = Utility.TextureFromPicture(graphics.GraphicsDevice, this.SourceModel.Skins[0].Textures[0].Picture.GetNearestPowerOfTwo());

			//this.Vertices = Array.ConvertAll<Quake.Files.Model.Vertex, VertexPositionNormalTexture>(this.SourceModel.Frames[0].Frames[0].Vertices, delegate(Quake.Files.Model.Vertex v) {
				//return new VertexPositionNormalTexture(Utility.ConvertPosition(v.Position.UnpackedPosition), Utility.ConvertPosition(v.Normal), Xna.Vector2.Zero);
			//});

			this.Triangles = new short[this.SourceModel.Triangles.Length * 3];
			this.Vertices = new VertexPositionNormalTexture[this.Triangles.Length * 3];

			for (int i = 0; i < this.SourceModel.Triangles.Length; ++i) {
				Quake.Files.Model.Triangle T = this.SourceModel.Triangles[i];
				Quake.Vector2[] TextureCoordinates = T.TextureCoordinates;

				for (int j = 0; j < 3; ++j) {
					Quake.Files.Model.Vertex V = this.SourceModel.Frames[0].Frames[0].Vertices[T.Vertices[j]];
					this.Triangles[i * 3 + j] = (short)(i * 3 + j);
					this.Vertices[i * 3 + j] = new VertexPositionNormalTexture(Utility.ConvertPosition(V.Position.UnpackedPosition), Utility.ConvertPosition(V.Normal), Utility.ConvertPosition(TextureCoordinates[j]));
				}				
			}

		}

		public void Unload(Microsoft.Xna.Framework.GraphicsDeviceManager graphics, Microsoft.Xna.Framework.Content.ContentManager content) {
			if (this.DiffuseTexture != null && !this.DiffuseTexture.IsDisposed) this.DiffuseTexture.Dispose();
		}


		#endregion

		#region IDrawable Members

		public void Draw(RenderController controller) {


			controller.WorldEffect.CurrentTechnique = controller.WorldEffect.Techniques["Model"];

			controller.WorldEffect.Begin();

			controller.VertexType = VertexPositionNormalTexture.VertexElements;
			controller.WorldEffect.Parameters["DiffuseTexture"].SetValue(this.DiffuseTexture);
			controller.WorldEffect.CommitChanges();

			foreach (EffectPass Pass in controller.WorldEffect.CurrentTechnique.Passes) {
				Pass.Begin();
				foreach (EntityPosition Pos in this.EntityPositions) {
					controller.WorldEffect.Parameters["WorldViewProj"].SetValue(Xna.Matrix.CreateFromYawPitchRoll(Pos.Angle.X, Pos.Angle.Y, Pos.Angle.Z) * Xna.Matrix.CreateTranslation(Pos.Position) * controller.BaseWorldViewProj);
					controller.WorldEffect.CommitChanges();
					controller.Graphics.GraphicsDevice.DrawUserIndexedPrimitives(PrimitiveType.TriangleList, this.Vertices, 0, this.Vertices.Length, this.Triangles, 0, this.Triangles.Length / 3);	
				}
				Pass.End();
			}

			controller.WorldEffect.End();
		}

		#endregion

	}
}
